package com.beifan.foxlibc.framework.spring.configuration;

import com.beifan.foxlibc.framework.utils.R;
import com.beifan.foxlibc.framework.utils.StringUtils;
import org.springframework.context.annotation.Configuration;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.servlet.mvc.method.annotation.ExceptionHandlerExceptionResolver;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * 全局异常拦截器
 *
 * @author airduck-buscentral
 * Create time 2022/4/18
 */
@Configuration
@ControllerAdvice
@ResponseBody
public class GlobalExceptionHandler extends ExceptionHandlerExceptionResolver
{

    /**
     * 业务异常捕获
     */
    @ExceptionHandler(value = Exception.class)
    public R<String> exceptionHandler(HttpServletRequest request, Exception e) {
        e.printStackTrace();

        // 返回异常信息
        String message = e.getMessage();
        if (message.startsWith("-C")) {
            int space = message.indexOf(" ");
            String code = message.substring(2, space);
            return R.fail(code, message.substring(space));
        }

        return R.fail(message);
    }

    /**
     * 参数校验异常捕获
     */
    @ExceptionHandler(value = MethodArgumentNotValidException.class)
    public R<String>  methodArgumentNotValidExceptionHandler(HttpServletRequest request, MethodArgumentNotValidException e) {
        String errorMessage = "";
        List<ObjectError> errors = e.getBindingResult().getAllErrors();
        for (ObjectError objectError : errors) {
            errorMessage = objectError.getDefaultMessage();
        }
        return R.fail(errorMessage);
    }

}
