package com.beifan.foxlibc.framework.utils;

import com.beifan.foxlibc.framework.utils.collection.Maps;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import static com.beifan.foxlibc.framework.utils.R.Status.S200;
import static com.beifan.foxlibc.framework.utils.R.Status.S500;

/**
 * @author lts
 * Create time 2022/4/15
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
public class R<T> {

    private String code;
    private String message;
    private Object returnData;

    public interface Status
    {
        String S200 = "0";
        String S500 = "500";
        String S401 = "401";
    }

    public static <T> R<T> ok() {
        return ok(null);
    }

    public static <T> R<T> ok(Object record) {
        return new R<T>(S200, "success", record);
    }

    /**
     * 自定义填充数据填充
     */
    public static <T> R<T> ok(String k1, Object v1) {
        return ok(Maps.ofMap(k1, v1));
    }

    public static <T> R<T> ok(String k1, Object v1, String k2, Object v2) {
        return ok(Maps.ofMap(k1, v1, k2, v2));
    }

    public static <T> R<T> ok(String k1, Object v1, String k2, Object v2, String k3, Object v3) {
        return ok(Maps.ofMap(k1, v1, k2, v2, k3, v3));
    }

    public static <T> R<T> ok(String k1, Object v1, String k2, Object v2, String k3, Object v3, String k4, Object v4) {
        return ok(Maps.ofMap(k1, v1, k2, v2, k3, v3, k4, v4));
    }

    public static <T> R<T> ok(String k1, Object v1, String k2, Object v2, String k3, Object v3, String k4, Object v4, String k5, Object v5) {
        return ok(Maps.ofMap(k1, v1, k2, v2, k3, v3, k4, v4, k5, v5));
    }

    public static <T> R<T> ok(String k1, Object v1, String k2, Object v2, String k3, Object v3, String k4, Object v4, String k5, Object v5, String k6, Object v6) {
        return ok(Maps.ofMap(k1, v1, k2, v2, k3, v3, k4, v4, k5, v5, k6, v6));
    }

    public static <T> R<T> fail(String code, String message) {
        return new R<T>(code, message, null);
    }

    public static <T> R<T> fail(String message) {
        return fail(S500, message);
    }

    /**
     * 结果是否成功
     */
    public static <T> boolean isSuccess(R<T> ret) {
        return ret != null && StringUtils.equals(ret.getCode(), S200);
    }

}
