package com.beifan.foxlibc.framework.utils.collection;

/* ************************************************************************
 *
 * Copyright (C) 2020 2B键盘 All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not useEnv this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * ************************************************************************/

/*
 * Creates on 2019/12/13
 */

import java.lang.reflect.Array;
import java.util.Comparator;
import java.util.List;

/**
 * 静态数组工具类
 *
 * @author lts
 */
public final class ArrayUtils
{

    public enum OP {FIRST, LAST}

    /**
     * 判断对象是否为数组
     */
    public static boolean isArray(Object o) {
        return o != null && o.getClass().isArray();
    }

    /**
     * append for array.
     *
     * @param array 需要追加的数组 | the array
     * @param value 需要追加的数据 | append value
     * @return false代表数组已经满了不能追加 | false express array full
     */
    public static <T> boolean append(T[] array, T value) {
        for (int i = 0; i < array.length; i++) {
            if (array[i] == null) {
                array[i] = value;
                return true;
            }
        }
        return false;
    }

    /**
     * 根据下标添加
     *
     * @param array 需要追加的数组 | the array
     * @param value 需要追加的数据 | append value
     * @param index 在哪个位置追加数据 | append by index
     * @return false代表数组已经满了不能追加 | false express array full
     */
    public static <T> boolean append(T[] array, T value, int index) {
        if (array.length < index)
            return false;
        array[index] = value;
        return false;
    }

    /**
     * 复制数组原始长度，并将数组扩容到capacity长度，使用null填充。
     *
     * @param a        数组对象
     * @param capacity 容量
     * @return 扩容后的数组
     */
    public static <T> T[] copyOf(T[] a, int capacity) {
        return java.util.Arrays.copyOf(a, capacity);
    }

    /**
     * 复制数组原始长度，并将数组扩容到capacity长度，使用null填充。
     *
     * @param original  源数组对象
     * @param newLength 新的数组长度
     * @param newType   新的数组类型
     * @return 扩容后的数组
     */
    public static <T, U> T[] copyOf(U[] original, int newLength, Class<? extends T[]> newType) {
        return java.util.Arrays.copyOf(original, newLength, newType);
    }

    /**
     * 数组转List，调用{@link Lists#ofList}方法转数组
     *
     * @param a 数组对象
     * @return 实例化的List对象
     */
    public static <E> List<E> asList(E[] a) {
        return Lists.ofList(a);
    }

    /**
     * 数组排序
     */
    public static <E> void sort(E[] a, Comparator<? super E> c) {
        java.util.Arrays.sort(a, c);
    }

    /**
     * 对象转数组, 传入的对象必须是数组转成的。
     */
    public static Object[] toArray(Object a) {
        Object[] objects = null;
        if (a != null) {
            if (isArray(a)) {
                int size = Array.getLength(a);
                objects = new Object[size];
                for (int i = 0; i < size; i++) {
                    objects[i] = Array.get(a, i);
                }
                return objects;
            } else if (a instanceof List) {
                List<?> list = (List<?>) a;
                int len = list.size();
                objects = new Object[len];
                for (int i = 0; i < len; i++) {
                    objects[i] = list.get(i);
                }
            }
        }
        return objects;
    }

    /**
     * 数组的{@code toString}方法
     */
    public static String toString(Object array) {
        if (!isArray(array)) {
            throw new Error("parameter object not array.");
        }
        Object[] array0 = toArray(array);
        StringBuilder str = new StringBuilder("[");
        for (Object o : array0) {
            str.append(o).append(",");
        }
        str = new StringBuilder(str.substring(0, str.length() - 1) + "]");
        return str.toString();
    }

    /**
     * 数组的{@code toString}方法
     */
    public static String toString(Object array, String delimiter) {
        if (!isArray(array)) {
            throw new Error("parameter object not array.");
        }
        Object[] array0 = toArray(array);
        StringBuilder str = new StringBuilder("[");
        for (Object o : array0) {
            str.append(o).append(delimiter);
        }
        str = new StringBuilder(str.substring(0, str.length() - 1) + "]");
        return str.toString();
    }

}
