package com.beifan.foxlibc.framework.utils.http;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.beifan.foxlibc.framework.utils.Assert;
import com.squareup.okhttp.*;
import lombok.extern.slf4j.Slf4j;

/**
 * @author lts
 * Create time 2022/4/17
 */
@Slf4j
public class HttpClients {

    private static final OkHttpClient httpClient = new OkHttpClient();

    /**
     * 发起post请求
     *
     * @param url       请求路径
     * @param paramBody 请求参数
     * @return          返回结果
     */
    public static String post(String url, Object paramBody)
    {
        String paramBodyValue = paramBody instanceof String ? String.valueOf(paramBody) : JSON.toJSONString(paramBody);
        Request requestBuilder = new Request.Builder()
                .url(url)
                .post(RequestBody.create(MediaType.parse("application/json"), paramBodyValue))
                .build();

        try {
            Response response = httpClient.newCall(requestBuilder).execute();

            String returnBody = response.body() == null ? "" : response.body().string();
            Assert.throwIfBool(response.isSuccessful(), "HTTP请求出错, URL: {}, ParamBody: {}, MESSAGE: {}",
                    url, paramBodyValue, returnBody);

            log.info("http请求返回结果: URL: {}, ParamBody: {}, ReturnBody: {}", url, paramBodyValue, returnBody);

            return returnBody;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 发起post请求
     *
     * @param url       请求路径
     * @param paramBody 请求参数
     * @param _class    将请求结果转换成对应的类
     * @return          返回结果
     */
    public static <T> T post(String url, Object paramBody, Class<T> _class)
    {
        JSONObject returnBody = JSONObject.parseObject(post(url, paramBody));
        return JSON.toJavaObject(returnBody, _class);
    }

}
