package com.beifan.foxlibc.framework.utils.io;

import com.beifan.foxlibc.framework.utils.Assert;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Objects;

/**
 * 文件工具
 *
 * @author lts
 * Create time 2022/4/25
 */
public class Files
{

    /**
     * 读取输入路径的内容
     */
    public static InputConverter fromPath(String path) throws IOException
    {
        return Readers.fromPath(path);
    }

    /**
     * 读取resource文件夹下的文件内容
     */
    public static InputConverter fromResource(String path) throws IOException
    {
        return Readers.fromResource(path);
    }

    /**
     * 创建文件夹
     * @param path 文件夹路径，如果前面的路径不存在就会创建
     */
    public static void mkdir(String path)
    {
        Assert.throwIfBool(new File(path).mkdirs(), "创建文件失败, 请检查当前用户是否有权限创建文件");
    }

    /**
     * 重新创建文件夹
     * @param path 文件夹路径，如果前面的路径不存在就会创建
     */
    public static void rmkdir(String path)
    {
        remove(path);
        mkdir(path);
    }

    /**
     * 创建文件
     */
    public static void makefile(String path, String content)
    {
        File file = new File(path);

        try (BufferedWriter writer = new BufferedWriter(new FileWriter(file))) {
            /* 创建文件 */
            if (!file.exists())
                Assert.throwIfBool(file.createNewFile(), "创建文件失败, 请检查当前用户是否有权限创建文件");

            writer.write(content);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 删除文件
     * @param file 文件
     */
    public static void remove(String file)
    {
        remove(new File(file));
    }

    /**
     * 删除文件
     * @param file 文件
     */
    public static void remove(File file)
    {
        if (!file.exists())
            return;

        if (file.isDirectory()) {
            for (File subf : Objects.requireNonNull(file.listFiles())) {
                remove(subf);
            }
        }

        Assert.throwIfBool(file.delete(), "{}文件或目录删除失败，请检查当前启动用户是否存在权限。", file.getName());
    }

    /**
     * 追加文件内容
     *
     * @param path 文件路径
     * @param ap   追加内容
     */
    public static void append(String path, String ap)
    {
        try (BufferedWriter writer = new BufferedWriter(new FileWriter(path))) {
            writer.write(ap);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

}
