package com.beifan.foxlibc.framework.utils.security;

import com.beifan.foxlibc.framework.utils.StringUtils;
import org.bouncycastle.jce.provider.BouncyCastleProvider;

import java.math.BigInteger;
import java.nio.charset.StandardCharsets;
import java.security.*;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;
import java.util.Base64;

/**
 * 加密、解密、密钥等工具包
 *
 * @author lts
 * Create time 2022/4/22
 */
public class DoubleCoder {

    /**
     * 将文本加密成md5
     */
    public static String MD5(String plaintext) {
        String ciphertext = null;
        try {
            MessageDigest md5 = MessageDigest.getInstance("MD5");
            byte[] digest = md5.digest(plaintext.getBytes(StandardCharsets.UTF_8));
            ciphertext = new BigInteger(1, digest).toString(16);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return ciphertext;
    }

    /**
     * 去除密钥中的开头和结尾以及换行符，并转成byte[]
     */
    private static byte[] INITKEY(String key) {
        if (key.contains("-----BEGIN PRIVATE KEY-----")) {
            key = key.substring(key.indexOf("-----BEGIN PRIVATE KEY-----") + 27);
        }
        if (key.contains("-----BEGIN PUBLIC KEY-----")) {
            key = key.substring(key.indexOf("-----BEGIN PUBLIC KEY-----") + 26);
        }
        if (key.contains("-----END PRIVATE KEY-----")) {
            key = key.substring(0, key.indexOf("-----END PRIVATE KEY-----"));
        }
        if (key.contains("-----END PUBLIC KEY-----")) {
            key = key.substring(0, key.indexOf("-----END PUBLIC KEY-----"));
        }
        key = key.replaceAll("\r\n", "");
        key = key.replaceAll("\n", "");
        return BASE64DECODE(StringUtils.mergeOneLine(key));
    }

    /**
     * 将文本反序列化成RS256公钥
     */
    public static PublicKey PUBLICKEYRSA256(String key) {
        try {
            /* 序列化公钥 */
            X509EncodedKeySpec pubKeySpec = new X509EncodedKeySpec(INITKEY(key));
            KeyFactory factory = KeyFactory.getInstance("RSA");
            return factory.generatePublic(pubKeySpec);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 将文本反序列化成RS256私钥
     */
    public static PrivateKey PRIVATERSA256(String key) {
        try {
            /* 序列化私钥 */
            PKCS8EncodedKeySpec  pubKeySpec = new PKCS8EncodedKeySpec (INITKEY(key));
            KeyFactory factory = KeyFactory.getInstance("RSA");
            return factory.generatePrivate(pubKeySpec);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 将文本反序列化成EC256公钥
     */
    public static PublicKey PUBLICKEYEC256(String key) {
        try {
            /* 序列化公钥 */
            Security.addProvider(new BouncyCastleProvider());
            KeyFactory keyFactory = KeyFactory.getInstance("ECDH", "BC");
            X509EncodedKeySpec pubX509 = new X509EncodedKeySpec(INITKEY(key));
            return keyFactory.generatePublic(pubX509);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 将文本反序列化成EC256私钥
     */
    public static PrivateKey PRIVATEEC256(String key) {
        try {
            /* 序列化私钥 */
            Security.addProvider(new BouncyCastleProvider());
            KeyFactory keyFactory = KeyFactory.getInstance("ECDH", "BC");
            PKCS8EncodedKeySpec devicePriKeySpec = new PKCS8EncodedKeySpec(key.getBytes());
            return keyFactory.generatePrivate(devicePriKeySpec);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public static void main(String[] args) {
        PRIVATEEC256("");
    }

    /**
     * base64解码
     */
    public static byte[] BASE64DECODE(String base64Text) {
        return Base64.getDecoder().decode(base64Text);
    }
}
