package com.beifan.foxlibc.modules.configuration;

import com.beifan.foxlibc.framework.utils.http.WebRequests;
import com.beifan.foxlibc.modules.controller.AuthenticationController;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;

/**
 * 统一Token校验
 *
 * @author lts
 * Create Time 2022/7/11
 */
@Component
public class ControllerInterceptor implements HandlerInterceptor {

    private final AuthenticationController authenticationController;

    public ControllerInterceptor(AuthenticationController authenticationController) {
        this.authenticationController = authenticationController;
    }

    @Override
    @SuppressWarnings("NullableProblems")
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception
    {
        if (handler instanceof HandlerMethod) {
            HandlerMethod handlerMethod = (HandlerMethod) handler;

            if (!handlerMethod.hasMethodAnnotation(NV_TOKEN.class)) {
                String authorization = WebRequests.getAuthorization();
                Map<String,String> headers = new HashMap<>();
                headers.put("Authorization",authorization);
                authenticationController.validToken(headers);
            }
        }
        return true;
    }

}
