package com.beifan.foxlibc.modules.pojo.model;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.beifan.foxlibc.framework.utils.collection.Lists;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import java.util.Collections;
import java.util.List;

/**
 * 数据底座返回结构体
 *
 * @author lts
 * Create Time 2022/7/1
 */
@Data
@Slf4j
public class VirtualDataSourceStructure {

    /* BEGIN ======================================================================
     *
     * {
     * --->    "code": "0",
     * --->    "returnData": {
     *             "listValues": [
     *                 {
     *                     "result": 1
     *                 }
     *             ],
     *             "needPage": 1,
     *             "columnSet": [],
     *             "submitID": null
     *         }
     * }
     *
     */
    private String code;
    private ReturnData returnData;

    @Data
    public static class ReturnData {
        public List<Object> listValues;
        public Integer needPage;
        public List<Object> columnSet;
        public String submitID;
    }
    /*                                                                            */
    /*                                                                            */
    /*                                                                            */
    /* END ====================================================================== */

    public boolean isSuccess()
    {
        return "0".equals(code);
    }

    /**
     * 应用认证
     */
    public boolean appAuthentication()
    {
        if (Lists.isEmpty(returnData.listValues))
            return false;

        return ((JSONObject) returnData.listValues.get(0)).getInteger("result") == 1;
    }

    /**
     * 用户认证
     */
    public String userAuthentication()
    {
        if (Lists.isEmpty(returnData.listValues))
            return null;

        return ((JSONObject) returnData.listValues.get(0)).getString("result");
    }

    /**
     * 序列化
     */
    public <T> T serialize(Class<T> _class)
    {
        return ((JSONObject) returnData.listValues.get(0)).toJavaObject(_class);
    }

    public <T> List<T> serializelist(Class<T> _class)
    {
        return JSONArray.parseArray(JSON.toJSONString(returnData.listValues)).toJavaList(_class);
    }

    @Override
    public String toString() {
        return "DataReturnValue{" +
                "code='" + code + '\'' +
                ", returnData=" + JSON.toJSONString(returnData) +
                '}';
    }
}
