package com.beifan.foxlibc.modules.service;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.beifan.foxlibc.framework.utils.Assert;
import com.beifan.foxlibc.modules.pojo.model.SystemSet;
import com.beifan.foxlibc.modules.pojo.model.User;
import com.beifan.foxlibc.modules.pojo.model.UserAuth;
import com.beifan.foxlibc.modules.pojo.model.UserLogininfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

import static com.alibaba.fastjson.JSON.toJSONString;

/**
 * @author lts
 * Create Time 2022/8/4
 */
@Service
public class UserServiceImplements implements UserService {

    @Autowired
    private JdbcTemplate jdbcTemplate;

    @Override
    public SystemSet systemSetting() {
        String sql = "SELECT * FROM t_system_set order by set_id desc limit 1";
        return JSONObject.parseObject(toJSONString(jdbcTemplate.queryForMap(sql))).toJavaObject(SystemSet.class);
    }

    @Override
    public boolean application(String appid, String secret) {
        String sql = "SELECT COUNT(1) as result FROM t_application  WHERE  app_show_id = ? AND app_show_secret = ?";
        Integer ret = jdbcTemplate.queryForObject(sql, Integer.class, appid, secret);
        return ret != null && ret != 0;
    }

    @Override
    public String sign(String username, String password) {
        String sql = "SELECT user_id FROM t_user  WHERE  user_name = ? AND user_pwd =?";
        return Assert.throwIfError(() -> {
            return jdbcTemplate.queryForObject(sql, String.class, username, password);
        }, "用户名或密码不正确");
    }

    @Override
    public String idsign(String userid, String password) {
        String sql = "SELECT user_id FROM t_user  WHERE  user_id = ? AND user_pwd =?";
        return Assert.throwIfError(() -> {
            return jdbcTemplate.queryForObject(sql, String.class, userid, password);
        }, "用户名或密码不正确");
    }

    @Override
    public String secsign(String userId, String password) {
        String sql = "SELECT user_id FROM t_user  WHERE  user_id = ? AND user_sec_pwd =?";
        return Assert.throwIfError(() -> {
            return jdbcTemplate.queryForObject(sql, String.class, userId, password);
        }, "用户名或密码不正确");
    }

    @Override
    public User userQuery(String userId) {
        String sql = "SELECT * FROM t_user where user_id = ?";
        return JSONObject.parseObject(toJSONString(jdbcTemplate.queryForMap(sql, userId))).toJavaObject(User.class);
    }


    @Override
    public List<UserAuth> authQuery(String userId) {
        return null;
    }

//    @Override
//    public void updateUserState(String userId, String token, String sate, String validDate) {
//        String sql = "update t_user set user_token = ?, user_status = ?, token_valid_time = ? where user_id = ?";
//        jdbcTemplate.update(sql, token, sate, validDate, userId);
//    }

    @Override
    public void saveUserLogininfo(UserLogininfo userLogininfo) {
        String sql = "INSERT INTO `t_user_token_cache` (`appid`,`userId`, `expireTime`, `sessionId`, `token`,`verificationCode`) VALUES (?,?,?,?, ?, ?);";
        jdbcTemplate.update(sql, userLogininfo.getAppid(), userLogininfo.getUserId(), userLogininfo.getExpireTime(), userLogininfo.getSessionId(), userLogininfo.getToken(), userLogininfo.getVerificationCode());
    }

    @Override
    public void updateUserLogininfo(UserLogininfo userLogininfo, String oldToken) {
        String sql = "UPDATE `t_user_token_cache` SET `token` = ?,`userId` = ?, `sessionId` = ?, `expireTime` = ?, `verificationCode` = ?,`codeExpireTime` = ? WHERE token = ?;";
        jdbcTemplate.update(sql, userLogininfo.getToken(), userLogininfo.getUserId(), userLogininfo.getSessionId(), userLogininfo.getExpireTime(), userLogininfo.getVerificationCode(), userLogininfo.getCodeExpireTime(), oldToken);
    }

    @Override
    public void deleteUserLogininfoByToken(String token) {
        String sql = "DELETE FROM `t_user_token_cache` WHERE token = ?";
        jdbcTemplate.update(sql, token);
    }

    @Override
    public void deleteUserLogininfoByUserId(String userid) {
        String sql = "DELETE FROM `t_user_token_cache` WHERE userId = ?";
        jdbcTemplate.update(sql, userid);
    }

    @Override
    public UserLogininfo queryUserLogininfo(String token) {
        String sql = "SELECT * FROM t_user_token_cache  WHERE  token = ?";
        return Assert.throwIfError(() -> {
            return JSONObject.parseObject(toJSONString(jdbcTemplate.queryForMap(sql, token))).toJavaObject(UserLogininfo.class);
        }, "token已过期");
    }

    @Override
    public List<UserLogininfo> queryUserLogininfosByUserId(String userId) {
        String sql = "SELECT * FROM t_user_token_cache  WHERE  userId = ?";
        return Assert.throwIfError(() -> {
            return jdbcTemplate.queryForList(sql, UserLogininfo.class, userId);
        }, "用户不存在");
    }

    @Override
    public UserLogininfo queryUserLogininfosByUserIdPS(String userId) {
        String sql = "SELECT * FROM t_user_token_cache  WHERE  userId = ? and not isnull(token)";
        return Assert.throwIfError(() -> {
            return JSONObject.parseObject(toJSONString(jdbcTemplate.queryForMap(sql, userId))).toJavaObject(UserLogininfo.class);
        }, "用户不存在");
    }

    @Override
    public void updateUserPwd(String userid, String pwd) {
        String sql = "UPDATE `t_user` SET user_pwd= ? WHERE user_id = ?";
        jdbcTemplate.update(sql, pwd, userid);
    }

    @Override
    public void updateUserSecPwd(String userid, String pwd) {
        String sql = "UPDATE `t_user` SET user_sec_pwd = ? WHERE user_id = ?";
        jdbcTemplate.update(sql, pwd, userid);
    }

    @Override
    public User queryUserByUsername(String username) {
        ArrayList<Object> objects = new ArrayList<>();
        objects.add(username);
        Object[] args = objects.toArray();
        return JSON.parseObject(toJSONString(jdbcTemplate.queryForMap("SELECT * FROM t_user WHERE user_name=? limit 1", args)), User.class);
    }

    @Override
    public void updateFailCount(String username, int failCount) {
        jdbcTemplate.update("UPDATE t_user SET fail_count = ? WHERE user_name = ?", failCount, username);
    }

    @Override
    public void updateStatus(String username, String status) {
        jdbcTemplate.update("UPDATE t_user SET user_status = ? WHERE user_name = ?", status, username);
    }
}
